@@ -50,8 +50,8 @@ module Network.TLS (
     clientHooks,
     clientSupported,
     clientDebug,
-    clientLimit,
     clientUseEarlyData,
+    clientUseECH,
 
     -- ** Server parameters
     ServerParams,
@@ -63,9 +63,9 @@ module Network.TLS (
     serverShared,
     serverSupported,
     serverDebug,
-    serverLimit,
     serverEarlyDataSize,
     serverTicketLifetime,
+    serverECHKey,
 
     -- ** Shared
     Shared,
@@ -75,6 +75,8 @@ module Network.TLS (
     sharedCAStore,
     sharedValidationCache,
     sharedHelloExtensions,
+    sharedECHConfigList,
+    sharedLimit,
 
     -- ** Client hooks
     ClientHooks,
@@ -125,6 +127,9 @@ module Network.TLS (
     debugPrintSeed,
     debugVersionForced,
     debugKeyLogger,
+    defaultKeyLogger,
+    debugError,
+    debugTraceKey,
 
     -- ** Limit parameters
     Limit,
@@ -224,6 +229,7 @@ module Network.TLS (
     infoTLS12Resumption,
     infoTLS13HandshakeMode,
     infoIsEarlyDataAccepted,
+    infoIsECHAccepted,
     ClientRandom,
     ServerRandom,
     unClientRandom,
@@ -299,6 +305,9 @@ module Network.TLS (
     clientUseMaxFragmentLength,
 ) where
 
+import Data.X509 (PrivKey (..), PubKey (..))
+import Data.X509.Validation hiding (HostName, defaultHooks)
+
 import Network.TLS.Backend (Backend (..), HasBackend (..))
 import Network.TLS.Cipher
 import Network.TLS.Compression (
@@ -318,6 +327,7 @@ import Network.TLS.Crypto (
  )
 import Network.TLS.Handshake.State (HandshakeMode13 (..))
 import Network.TLS.Hooks
+import Network.TLS.Imports
 import Network.TLS.Measurement
 import Network.TLS.Parameters
 import Network.TLS.Session
@@ -341,12 +351,8 @@ import Network.TLS.Struct13 (Handshake13)
 import Network.TLS.Types
 import Network.TLS.X509
 
-import Data.ByteString as B
-import Data.X509 (PrivKey (..), PubKey (..))
-import Data.X509.Validation hiding (HostName, defaultHooks)
-
 {-# DEPRECATED Bytes "Use Data.ByteString.Bytestring instead of Bytes." #-}
-type Bytes = B.ByteString
+type Bytes = ByteString
 
 -- | Getting certificates from a client, if any.
 --   Note that the certificates are not sent by a client
