@@ -1,22 +1,28 @@
+{-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE MultiWayIf #-}
+{-# LANGUAGE OverloadedLists #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
 
 module Main where
 
 import Control.Concurrent
+import qualified Control.Exception as E
+import qualified Data.ByteString.Base16 as BS16
 import qualified Data.ByteString.Char8 as C8
-import Data.Default.Class (def)
 import Data.IORef
+import Data.List.NonEmpty (NonEmpty)
+import qualified Data.List.NonEmpty as NE
 import Data.X509.CertificateStore
 import Network.Run.TCP
 import Network.Socket
-import Network.TLS
+import Network.TLS hiding (is0RTTPossible)
+import Network.TLS.ECH.Config
+import Network.TLS.Internal (makeCipherShowPretty)
 import System.Console.GetOpt
 import System.Environment
 import System.Exit
 import System.X509
-import qualified UnliftIO.Exception as E
 
 import Client
 import Common
@@ -33,6 +39,13 @@ data Options = Options
     , opt0RTT :: Bool
     , optRetry :: Bool
     , optVersions :: [Version]
+    , optALPN :: String
+    , optCertFile :: Maybe FilePath
+    , optKeyFile :: Maybe FilePath
+    , optECHConfigFile :: Maybe FilePath
+    , optTraceKey :: Bool
+    , optIPv4Only :: Bool
+    , optIPv6Only :: Bool
     }
     deriving (Show)
 
@@ -42,13 +55,20 @@ defaultOptions =
         { optDebugLog = False
         , optShow = False
         , optKeyLogFile = Nothing
-        , optGroups = supportedGroups def
+        , optGroups = supportedGroups defaultSupported
         , optValidate = False
         , optVerNego = False
         , optResumption = False
         , opt0RTT = False
         , optRetry = False
-        , optVersions = supportedVersions def
+        , optVersions = supportedVersions defaultSupported
+        , optALPN = "http/1.1"
+        , optCertFile = Nothing
+        , optKeyFile = Nothing
+        , optECHConfigFile = Nothing
+        , optTraceKey = False
+        , optIPv4Only = False
+        , optIPv6Only = False
         }
 
 usage :: String
@@ -106,13 +126,48 @@ options =
         ["tls13"]
         (NoArg (\o -> o{optVersions = [TLS13]}))
         "use TLS 1.3"
+    , Option
+        ['a']
+        ["alpn"]
+        (ReqArg (\a o -> o{optALPN = a}) "<alpn>")
+        "set ALPN"
+    , Option
+        ['c']
+        ["cert"]
+        (ReqArg (\fl o -> o{optCertFile = Just fl}) "<file>")
+        "certificate file"
+    , Option
+        ['k']
+        ["key"]
+        (ReqArg (\fl o -> o{optKeyFile = Just fl}) "<file>")
+        "key file"
+    , Option
+        []
+        ["ech-config"]
+        (ReqArg (\fl o -> o{optECHConfigFile = Just fl}) "<file>")
+        "ECH config file"
+    , Option
+        []
+        ["trace-key"]
+        (NoArg (\o -> o{optTraceKey = True}))
+        "Trace transcript hash"
+    , Option
+        ['4']
+        []
+        (NoArg (\o -> o{optIPv4Only = True, optIPv6Only = False}))
+        "IPv4 only"
+    , Option
+        ['6']
+        []
+        (NoArg (\o -> o{optIPv6Only = True, optIPv4Only = False}))
+        "IPv6 only"
     ]
 
 showUsageAndExit :: String -> IO a
 showUsageAndExit msg = do
     putStrLn msg
     putStrLn $ usageInfo usage options
-    putStrLn $ "  <groups> = " ++ (intercalate "," (map fst namedGroups))
+    putStrLn $ "  <groups> = " ++ intercalate "," (map fst namedGroups)
     exitFailure
 
 clientOpts :: [String] -> IO (Options, [String])
@@ -129,14 +184,24 @@ main = do
         [] -> showUsageAndExit usage
         _ : [] -> showUsageAndExit usage
         h : p : [] -> return (h, p, ["/"])
-        h : p : ps -> return (h, p, C8.pack <$> ps)
+        h : p : ps -> return (h, p, C8.pack <$> NE.fromList ps)
     when (null optGroups) $ do
         putStrLn "Error: unsupported groups"
         exitFailure
-    ref <- newIORef Nothing
+    let onCertReq = \_ -> case optCertFile of
+            Just certFile -> case optKeyFile of
+                Just keyFile -> do
+                    Right (!cc, !priv) <- credentialLoadX509 certFile keyFile
+                    return $ Just (cc, priv)
+                _ -> return Nothing
+            _ -> return Nothing
+    ref <- newIORef []
     let debug
             | optDebugLog = putStrLn
             | otherwise = \_ -> return ()
+        traceKey
+            | optTraceKey = putStrLn
+            | otherwise = \_ -> return ()
         showContent
             | optShow = C8.putStr
             | otherwise = \_ -> return ()
@@ -144,23 +209,40 @@ main = do
             Aux
                 { auxAuthority = host
                 , auxPort = port
-                , auxDebug = debug
+                , auxDebugPrint = debug
                 , auxShow = showContent
                 , auxReadResumptionData = readIORef ref
                 }
     mstore <-
         if optValidate then Just <$> getSystemCertificateStore else return Nothing
-    let keyLog = getLogger optKeyLogFile
-        groups
-            | optRetry = FFDHE8192 : optGroups
-            | otherwise = optGroups
-        cparams = getClientParams optVersions host port groups (smIORef ref) mstore keyLog
-    runClient opts cparams aux paths
+    echConfList <- case optECHConfigFile of
+        Nothing -> return []
+        Just ecnff ->
+            loadECHConfigList ecnff `E.catch` \(E.SomeException _) -> do
+                putStrLn $ ecnff ++ " is broken"
+                exitFailure
+    let cparams =
+            getClientParams
+                opts
+                host
+                port
+                (smIORef ref)
+                mstore
+                onCertReq
+                echConfList
+                debug
+                traceKey
+        client
+            | optALPN == "dot" = clientDNS
+            | otherwise = clientHTTP11
+    makeCipherShowPretty
+    runClient opts client cparams aux paths
 
-runClient :: Options -> ClientParams -> Aux -> [ByteString] -> IO ()
-runClient opts@Options{..} cparams aux@Aux{..} paths = do
-    auxDebug "------------------------"
-    (info1, msd) <- runTLS cparams aux $ \ctx -> do
+runClient
+    :: Options -> Cli -> ClientParams -> Aux -> NonEmpty ByteString -> IO ()
+runClient opts@Options{..} client cparams aux@Aux{..} paths = do
+    auxDebugPrint "------------------------"
+    (info1, msd) <- runTLS opts cparams aux $ \ctx -> do
         i1 <- getInfo ctx
         when optDebugLog $ printHandshakeInfo i1
         client aux paths ctx
@@ -171,7 +253,7 @@ runClient opts@Options{..} cparams aux@Aux{..} paths = do
             if isResumptionPossible msd
                 then do
                     let cparams2 = modifyClientParams cparams msd False
-                    info2 <- runClient2 opts cparams2 aux paths
+                    info2 <- runClient2 opts client cparams2 aux paths
                     if infoVersion info1 == TLS12
                         then do
                             if infoTLS12Resumption info2
@@ -196,7 +278,7 @@ runClient opts@Options{..} cparams aux@Aux{..} paths = do
             if is0RTTPossible info1 msd
                 then do
                     let cparams2 = modifyClientParams cparams msd True
-                    info2 <- runClient2 opts cparams2 aux paths
+                    info2 <- runClient2 opts client cparams2 aux paths
                     if infoTLS13HandshakeMode info2 == Just RTT0
                         then do
                             putStrLn "Result: (Z) 0-RTT ... OK"
@@ -217,22 +299,22 @@ runClient opts@Options{..} cparams aux@Aux{..} paths = do
                     exitFailure
         | otherwise -> do
             putStrLn "Result: (H) handshake ... OK"
-            let malpn = (snd <$> msd) >>= sessionALPN
-            when (malpn == Just "http/1.1") $
+            when (optALPN == "http/1.1") $
                 putStrLn "Result: (1) HTTP/1.1 transaction ... OK"
             exitSuccess
 
 runClient2
     :: Options
+    -> Cli
     -> ClientParams
     -> Aux
-    -> [ByteString]
+    -> NonEmpty ByteString
     -> IO Information
-runClient2 Options{..} cparams aux@Aux{..} paths = do
+runClient2 opts@Options{..} client cparams aux@Aux{..} paths = do
     threadDelay 100000
-    auxDebug "<<<< next connection >>>>"
-    auxDebug "------------------------"
-    runTLS cparams aux $ \ctx -> do
+    auxDebugPrint "<<<< next connection >>>>"
+    auxDebugPrint "------------------------"
+    runTLS opts cparams aux $ \ctx -> do
         if opt0RTT
             then do
                 void $ client aux paths ctx
@@ -246,81 +328,126 @@ runClient2 Options{..} cparams aux@Aux{..} paths = do
                 return i
 
 runTLS
-    :: ClientParams
+    :: Options
+    -> ClientParams
     -> Aux
     -> (Context -> IO a)
     -> IO a
-runTLS cparams Aux{..} action =
-    runTCPClient auxAuthority auxPort $ \sock -> do
-        E.bracket (contextNew sock cparams) bye $ \ctx -> do
-            handshake ctx
-            action ctx
+runTLS Options{..} cparams Aux{..} action =
+    runTCPClientWithSettings settings auxAuthority auxPort $ \sock -> do
+        ctx <- contextNew sock cparams
+        when optDebugLog $
+            contextHookSetLogging
+                ctx
+                defaultLogging
+                    { loggingPacketSent = putStrLn . (">> " ++)
+                    , loggingPacketRecv = putStrLn . ("<< " ++)
+                    --                    , loggingIOSent = \bs -> putStrLn $ "}} " ++ showBytesHex bs
+                    --                    , loggingIORecv = \hd bs -> putStrLn $ "{{ " ++ show hd ++ " " ++ showBytesHex bs
+                    }
+        handshake ctx
+        r <- action ctx
+        bye ctx
+        return r
+  where
+    select addrs
+        | optIPv4Only = case NE.filter (\ai -> addrFamily ai == AF_INET) addrs of
+            [] -> error "IPv4 address is not available"
+            ai : _ -> ai
+        | optIPv6Only = case NE.filter (\ai -> addrFamily ai == AF_INET6) addrs of
+            [] -> error "IPv6 address is not available"
+            ai : _ -> ai
+        | otherwise = NE.head addrs
+    settings =
+        defaultSettings
+            { settingsSelectAddrInfo = select
+            }
 
 modifyClientParams
-    :: ClientParams -> Maybe (SessionID, SessionData) -> Bool -> ClientParams
-modifyClientParams cparams wantResume early =
+    :: ClientParams -> [(SessionID, SessionData)] -> Bool -> ClientParams
+modifyClientParams cparams ts early =
     cparams
-        { clientWantSessionResume = wantResume
+        { clientWantSessionResumeList = ts
         , clientUseEarlyData = early
         }
 
 getClientParams
-    :: [Version]
+    :: Options
     -> HostName
     -> ServiceName
-    -> [Group]
     -> SessionManager
     -> Maybe CertificateStore
+    -> OnCertificateRequest
+    -> ECHConfigList
+    -> (String -> IO ())
     -> (String -> IO ())
     -> ClientParams
-getClientParams vers serverName port groups sm mstore keyLog =
+getClientParams Options{..} serverName port sm mstore onCertReq echConfList printError traceKey =
     (defaultParamsClient serverName (C8.pack port))
         { clientSupported = supported
         , clientUseServerNameIndication = True
         , clientShared = shared
         , clientHooks = hooks
         , clientDebug = debug
+        , clientUseECH = not (null echConfList)
         }
   where
+    groups
+        | optRetry = FFDHE8192 : optGroups
+        | otherwise = optGroups
     shared =
-        def
+        defaultShared
             { sharedSessionManager = sm
-            , sharedCAStore = case mstore of
-                Just store -> store
-                Nothing -> mempty
+            , sharedCAStore = fromMaybe mempty mstore
             , sharedValidationCache = validateCache
+            , sharedLimit =
+                defaultLimit
+                    { limitRecordSize = Just 8192
+                    }
+            , sharedECHConfigList = echConfList
             }
     supported =
-        def
-            { supportedVersions = vers
+        defaultSupported
+            { supportedVersions = optVersions
             , supportedGroups = groups
             }
     hooks =
-        def
-            { onSuggestALPN = return $ Just ["http/1.1"]
+        defaultClientHooks
+            { onSuggestALPN = return $ Just [C8.pack optALPN]
+            , onCertificateRequest = onCertReq
             }
     validateCache
-        | isJust mstore = def
+        | isJust mstore = sharedValidationCache defaultShared
         | otherwise =
             ValidationCache
                 (\_ _ _ -> return ValidationCachePass)
                 (\_ _ _ -> return ())
     debug =
-        def
-            { debugKeyLogger = keyLog
+        defaultDebugParams
+            { debugKeyLogger = getLogger optKeyLogFile
+            , debugError = printError
+            , debugTraceKey = traceKey
             }
 
-smIORef :: IORef (Maybe (SessionID, SessionData)) -> SessionManager
+smIORef :: IORef [(SessionID, SessionData)] -> SessionManager
 smIORef ref =
     noSessionManager
-        { sessionEstablish = \sid sdata -> writeIORef ref (Just (sid, sdata)) >> return Nothing
+        { sessionEstablish = \sid sdata ->
+            modifyIORef' ref (\xs -> (sid, sdata) : xs)
+                >> printTicket sid sdata
+                >> return Nothing
         }
 
-isResumptionPossible :: Maybe (SessionID, SessionData) -> Bool
-isResumptionPossible = isJust
+printTicket :: SessionID -> SessionData -> IO ()
+printTicket sid sdata = do
+    C8.putStr $ "Ticket: " <> C8.take 16 (BS16.encode sid) <> "..., "
+    putStrLn $ "0-RTT: " <> if sessionMaxEarlyDataSize sdata > 0 then "OK" else "NG"
 
-is0RTTPossible :: Information -> Maybe (SessionID, SessionData) -> Bool
-is0RTTPossible _ Nothing = False
-is0RTTPossible info (Just (_, sd)) =
+isResumptionPossible :: [(SessionID, SessionData)] -> Bool
+isResumptionPossible = not . null
+
+is0RTTPossible :: Information -> [(SessionID, SessionData)] -> Bool
+is0RTTPossible _ [] = False
+is0RTTPossible info xs =
     infoVersion info == TLS13
-        && sessionMaxEarlyDataSize sd > 0
+        && any (\(_, sd) -> sessionMaxEarlyDataSize sd > 0) xs
